// @flow
import 'rc-calendar/assets/index.css';
import React from 'react';
import Calendar from 'rc-calendar';
import DatePicker from 'rc-calendar/lib/Picker';
import zhCN from 'rc-calendar/lib/locale/zh_CN';
import 'rc-time-picker/assets/index.css';
import TimePickerPanel from 'rc-time-picker/lib/Panel';

import moment from 'moment';

type Props = {
  defaultValue: moment,
  format: string,
  disabled: boolean,
  placeholder: string,
  onChange: Function,
  onOk: Function,
  disabledDate: Function,
}

class DatePickerCon extends React.PureComponent<Props> {
  state = {
    value: this.props.defaultValue,
  }

  onChange = (value) => {
    this.setState({
      value,
    }, () => {
      this.props.onChange(value);
    });
  }

  onOk = (value) => {
    this.props.onOk(value);
  }

  render() {
    const { value } = this.state;
    const {
      disabledDate, placeholder, disabled, format,
    } = this.props;
    const timePickerElement = <TimePickerPanel defaultValue={moment('00:00', 'HH:mm')} />;
    const calendar = (
      <Calendar
        locale={zhCN}
        style={{ zIndex: 1000 }}
        timePicker={timePickerElement}
        dateInputPlaceholder=""
        format={format}
        showDateInput
        disabledDate={disabledDate}
      />
    );
    return (
      <div>
        <DatePicker
          disabled={disabled}
          animation="slide-up"
          calendar={calendar}
          value={value}
          onChange={this.onChange}
          onOk={this.onOk}
        >
          {
            () => (
              <span>
                <input
                  disabled={disabled}
                  placeholder={placeholder || ''}
                  readOnly
                  tabIndex="-1"
                  className="date-picker-input"
                  value={value ? moment(value).format('YYYY-MM-DD HH:mm') : ''}
                />
              </span>
            )
          }
        </DatePicker>
        <style jsx>
          {`
          .date-picker-input {
            -webkit-box-sizing: border-box;
            box-sizing: border-box;
            position: relative;
            padding: 4px 11px;
            height: 32px;
            line-height: 1.5;
            color: rgba(0,0,0,.65);
            background-color: #fff;
            background-image: none;
            border: 1px solid #d9d9d9;
            border-radius: 4px;
            -webkit-transition: all .3s;
            transition: all .3s;
            outline: none;
          }
          .date-picker-input:hover{
            border: 1px solid #009edc;
          }
          .date-picker-input:disabled, .date-picker-input:disabled:hover { 
            border: 1px solid #eee;
          }
        `}
        </style>
      </div>
    );
  }
}
export default DatePickerCon;
