// @flow
import React from 'react';
import DatePicker from 'react-datepicker';
import moment from 'moment';

type Props = {
  onDateChange: Function,
  start: string,
  end: string,
}
const hasTime = (value => (value.start.length > 4 && value.end.length > 4));

const formateDate = (start, end) => ({ start: moment(start).format('YYYY-MM-DD'), end: moment(end).format('YYYY-MM-DD') });
class DateRange extends React.PureComponent<Props> {
  state = {
    start: undefined,
    end: undefined,
  }

  static getDerivedStateFromProps(nextProps: Props, prevState) {
    const oldDate = formateDate(prevState.start, prevState.end);
    if (hasTime(nextProps) && (nextProps.start !== oldDate.start || nextProps.end !== oldDate.end)) {
      return { start: moment(nextProps.start), end: moment(nextProps.end) };
    }
    return null;
  }

  changeDate(value, keyName) {
    this.setState({ [keyName]: value }, this.returnValues);
  }

  returnValues() {
    const range = formateDate(this.state.start, this.state.end);
    console.log(this.props.start);
    console.log(range);
    this.props.onDateChange(range);
  }


  render() {
    const { start, end } = this.state;
    return (
      <div className="inline-date">
        <div className="service-from">
          <DatePicker
            id="service-cycle"
            locale="zh-cn"
            placeholderText="开始日期"
            dateFormat="YYYY-MM-DD"
            selected={start}
            selectsStart
            startDate={start}
            endDate={end}
            maxDate={end}
            onChange={(e) => { this.changeDate(e, 'start'); }}
          />
        </div>
        <span className="link-line">&ensp;&ensp;~&ensp;&ensp;</span>
        <div className="service-to">
          <DatePicker
            id="service-cycle"
            locale="zh-cn"
            placeholderText="结束日期"
            dateFormat="YYYY-MM-DD"
            selected={end}
            selectsEnd
            startDate={start}
            endDate={end}
            minDate={start}
            onChange={(e) => { this.changeDate(e, 'end'); }}
          />
        </div>
        <style jsx>
          {`
          .inline-date {
            display: flex;
          }
        `}
        </style>
      </div>
    );
  }
}
export default DateRange;
