// @flow
import React from 'react';

type Props = {
  onStatusChange: Function,
  activeKey: string,
  steps: Array<Object>,
}
type State = {
  currentStep: number,
}


const isProcessed = (index, activeIndex, currentStep) => {
  if (index === currentStep) {
    return 'step-wraper step-wraper-active';
  }
  if (index > activeIndex) {
    return 'step-wraper';
  }
  return 'step-wraper step-wraper-complete';
};

class Step extends React.PureComponent<Props, State> {
  state = {
    currentStep: this.props.steps.findIndex(item => item.key === this.props.activeKey),
  }

  static getDerivedStateFromProps(nextProps, prevState) {
    const currentStep = nextProps.steps.findIndex(item => item.key === nextProps.activeKey);
    if (currentStep !== prevState.currentStep) {
      return { currentStep };
    }
    return null;
  }

  handleStatusChange = (index: number, key, activeKeyIndex) => {
    if (index > activeKeyIndex) return;
    this.setState({ currentStep: index });
    this.props.onStatusChange(key);
  }


  render() {
    const { activeKey, steps } = this.props;
    const { currentStep } = this.state;
    const activeKeyIndex = steps.findIndex(item => item.key === activeKey);
    return (
      <div className="container">
        <div className="line" />
        {
          /* eslint-disable */
          steps.map((item, index, arr) => (
            <div
              className={isProcessed(index, activeKeyIndex, currentStep)}
              style={index > activeKeyIndex ? { cursor: 'not-allowed' } : {}}
              key={item.key}
              onClick={() => { this.handleStatusChange(index, item.key, activeKeyIndex); }}
            >
              <div className="step">
                {index + 1}
              </div>
              <div className="step-label">{item.label}</div>
            </div>
          ))
        }
        <style jsx>{`
          .container {
            display: inline-block;
            position: relative;
          }
          .line {
            position: absolute;
            top: 12px;
            height: 1px;
            left: 65px;
            width: calc(100% - 120px);
            background-color: rgba(153,153,153,.3);
          }
          .step-wraper {
            min-width: 132px;
            float: left;
            margin-right: 60px;
            position: relative;
            z-index: 2;
            cursor: pointer;
          }
          .step {
            margin: 0 auto;
            border-radius: 50%;
            background-color: #cccccc;
            width: 23px;
            line-height: 23px;
            text-align: center;
            color: #ffffff;
          }
          .step-label {
            color: #999999;
            line-height: 30px;
            text-align: center;
          }
          .step-wraper-complete .step{
            background-color: #666666;
          }
          .step-wraper-complete .step-label{
            color: #666666;
          }
          .step-wraper-active .step{
            background-color: #009EDC;
          }
          .step-wraper-active .step-label{
            color: #009EDC;
          }
          .step-wraper:last-child {
            margin-right: 0;
          }
        `}
        </style>
      </div>
    );
  }
}
export default Step;
