// @flow
import React, { type ComponentType } from 'react';
import { connect } from 'react-redux';
import { withRouter } from 'next/router';
import ExceptionPage from '../ExceptionPage';
// import Error from 'next/error';

import { type User } from '../../store/user';

type Props = {
  user: User,
  loading: boolean,
}

const mapStateToProps = state => ({
  user: state.user,
  loading: state.persist.loading,
});

const withAuthorization = (allowedRoles: Array<string>) => (
  (WrappedComponent: ComponentType<any>) => {
    class WithAuthorization extends React.PureComponent<Props, State> {
      render() {
        const { user: { role }, loading } = this.props;
        if (loading) {
          // TODO: loading component ?
          return false;
        }
        if (allowedRoles.includes(role)) {
          return <WrappedComponent {...this.props} />;
        }
        return (
          <div className="container">
            <ExceptionPage page="403" />
            <style jsx>
              {`
                .container {
                  height: calc(100vh - 60px);
                  width: 100vw;
                  display: flex;
                }
                `}
            </style>
          </div>
        );
      }
    }

    return withRouter(connect(mapStateToProps, {})(WithAuthorization));
  }
);

export default withAuthorization;
